#ifndef COMPRESSOR_H_
#define COMPRESSOR_H_

#define COMPRESSOR_PRIORITY 90

#include "Relay.h"
#include "Utility.h"

class DigitalInput;

/**
 * Compressor object.
 * The Compressor object is designed to handle the operation of the compressor, pressure sensor and
 * relay for a FIRST robot pneumatics system. The Compressor object starts a task which runs in the
 * backround and periodically polls the pressure sensor and operates the relay that controls the
 * compressor.
 */ 
class Compressor: public SensorBase
{
public:
	Compressor(unsigned pressureSwitchChannel, unsigned compressorRelayChannel);
	Compressor(unsigned pressureSwitchSlot, unsigned pressureSwitchChannel,
				unsigned compresssorRelaySlot, unsigned compressorRelayChannel);
	~Compressor(void);

	void Start(void);
	void Stop(void);
	bool Enabled(void);
	unsigned GetPressureSwitchValue(void);
	void SetRelayValue(Relay::Value relayValue);

private:
	void InitCompressor(unsigned pressureSwitchSlot, unsigned pressureSwitchChannel,
					unsigned compresssorRelaySlot, unsigned compressorRelayChannel);

	DigitalInput *m_pressureSwitch;
	Relay *m_relay;
	bool m_enabled;
	int m_taskID;
};

#endif

