#ifndef CPPCOUNTER_BASE_H_
#define CPPCOUNTER_BASE_H_

/**
 * Class for counting the number of ticks on a digital input channel.
 * Encoders, Gear tooth sensors, and counters should all subclass this so it can be used to
 * build more advanced classes for control and driving.
 */
class CounterBase
{
public:
	virtual void Start(void) = 0;
	virtual signed int Get(void) = 0;
	virtual void Reset(void) = 0;
	virtual void Stop(void) = 0;
	virtual unsigned long GetPeriod(void) = 0;
	virtual void SetMaxPeriod(unsigned long maxPeriod) = 0;
	virtual bool GetStopped(void) = 0;
	virtual bool GetDirection(void) = 0;
};

#endif
