#ifndef DIGITAL_INPUT_H_
#define DIGITAL_INPUT_H_

class DigitalModule;

#include "DigitalSource.h"

/**
 * Class to read a digital input.
 * This class will read digital inputs and return the current value on the channel. Other devices
 * such as encoders, gear tooth sensors, etc. that are implemented elsewhere will automatically
 * allocate digital inputs and outputs as required. This class is only for devices like switches
 * etc. that aren't implemented anywhere else.
 */
class DigitalInput : public DigitalSource {
public:
	DigitalInput(unsigned channel);
	DigitalInput(unsigned slot, unsigned channel);
	~DigitalInput(void);
	unsigned Get(void);
	unsigned GetChannel(void);

	// Digital Source Interface
	virtual unsigned GetChannelForRouting(void);
	virtual unsigned GetModuleForRouting(void);
	virtual bool GetAnalogTriggerForRouting(void);
	
	// Interruptable Interface
	virtual void RequestInterrupts(tInterruptHandler handler); ///< Asynchronus handler version.
	virtual void RequestInterrupts(void);		///< Synchronus Wait version.
	void SetUpSourceEdge(bool risingEdge, bool fallingEdge);

private:
	void InitDigitalInput(unsigned slot, unsigned channel);
	unsigned m_channel;
	DigitalModule *m_module;
	bool m_lastValue;
};

#endif

