#ifndef __DRIVER_STATION_H__
#define __DRIVER_STATION_H__

#include "SensorBase.h"

struct FRCControlData;
class AnalogChannel;

/**
 * Provide access to the network communication data to / from the Driver Station.
 */
class DriverStation : public SensorBase
{
public:
	enum Alliance {kRed, kBlue, kInvalid};

	virtual ~DriverStation(void);
	static DriverStation *GetInstance(void);

	static const unsigned kBatterySlot = 1;
	static const unsigned kBatteryChannel = 8;
	static const unsigned kJoystickPorts = 4;
	static const unsigned kJoystickAxes = 6;

	float GetStickAxis(unsigned stick, unsigned axis);
	short GetStickButtons(unsigned stick);

	float GetAnalogIn(unsigned channel);
	bool GetDigitalIn(unsigned channel);
	void SetDigitalOut(unsigned channel, bool value);
	bool GetDigitalOut(unsigned channel);

	bool IsDisabled(void);
	bool IsAutonomous(void);
	bool IsOperatorControl(void);

	unsigned GetPacketNumber(void);
	Alliance GetAlliance(void);
	unsigned GetLocation(void);

	float GetBatteryVoltage(void);

protected:
	DriverStation(void);

	void GetData(void);
	void SetData(void);

private:
	static void InitTask(DriverStation *ds);
	static DriverStation *m_instance;
	static const int kUpdatePeriod_ms = 50;

	void Task(void);

	struct FRCControlData *m_controlData;
	char *m_userControl;
	char *m_userStatus;
	unsigned char m_digitalOut;
	AnalogChannel *m_batteryChannel;
	int m_taskID;
};

#endif

