#ifndef QUAD_ENCODER_H_
#define QUAD_ENCODER_H_

#include "ChipObject.h"
#include "CounterBase.h"

/**
 * Class to read quad encoders.
 * Quadrature encoders are devices that count shaft rotation and can sense direction. The output of
 * the QuadEncoder class is an integer that can count either up or down, and can go negative for
 * reverse direction counting. When creating QuadEncoders, a direction is supplied that changes the
 * sense of the output to make code more readable if the encoder is mounted such that forward movement
 * generates negative values. Quadrature encoders have two digital outputs, an A Channel and a B Channel
 * that are out of phase with each other to allow the FPGA to do direction sensing. 
 */
class Encoder: public SensorBase, public CounterBase
{
public:
	Encoder(unsigned aChannel, unsigned bChannel, bool reverseDirection=false);
	Encoder(unsigned aSlot, unsigned aChannel, unsigned bSlot, unsigned _bChannel, bool reverseDirection=false);
	Encoder(DigitalSource *aSource, DigitalSource *bSource, bool reverseDirection=false);
	virtual ~Encoder(void);

	// CounterBase interface
	void Start(void);
	signed int Get(void);
	void Reset(void);
	void Stop(void);
	unsigned long GetPeriod(void);
	void SetMaxPeriod(unsigned long maxPeriod);
	bool GetStopped(void);
	bool GetDirection(void);

private:
	void InitEncoder(bool _reverseDirection);

	DigitalSource *m_aSource;		// the A phase of the quad encoder
	DigitalSource *m_bSource;		// the B phase of the quad encoder
	bool m_allocatedASource;     // was the A source allocated locally?
	bool m_allocatedBSource;     // was the B source allocated locally?
	tEncoder* m_encoder;
	unsigned char m_index;
};

#endif

