#include "Resource.h"
#include <iostream.h>
#include "Utility.h"
#include "WPIStatus.h"

Resource *Resource::m_resourceList = NULL;

/**
 * Allocate storage for a new instance of Resource.
 * Allocate a bool array of values that will get initialized to indicate that no resources
 * have been allocated yet. The indicies of the resources are 0..size-1.
 */
Resource::Resource(unsigned elements)
{
	m_size = elements;
	m_isAllocated = new bool[m_size];
	for (unsigned i=0; i < m_size; i++)
		m_isAllocated[i] = false;
	m_nextResource = m_resourceList;
	m_resourceList = this;
}

/*static*/ void Resource::CreateResourceObject(Resource **r, unsigned elements)
{
	if (*r == NULL)
		*r = new Resource(elements);
}

/**
 * Delete the allocated array or resources.
 * This happens when the module is unloaded (provided it was statically allocated).
 */
Resource::~Resource(void)
{
	delete[] m_isAllocated;
}

/**
 * Allocate a resource.
 * When a resource is requested, mark it allocated. In this case, a free resource value
 * within the range is located and returned after it is marked allocated.
 */
unsigned Resource::Allocate(void)
{
	for (unsigned i=0; i < m_size; i++)
	{
		if ( ! m_isAllocated[i])
		{
			m_isAllocated[i] = true;
			return i;
		}
	}
	wpi_fatal(NoAvailablePorts);
	return 0;
}

/**
 * Allocate a specific resource value.
 * The user requests a specific resource value, i.e. channel number and it is verified
 * unallocated, then returned.
 */
unsigned Resource::Allocate(unsigned index)
{
	if (index >= m_size)
	{
		wpi_fatal(IndexOutOfRange);
		return 0;
	}
	if ( m_isAllocated[index] )
	{
		wpi_fatal(ResourceAlreadyAllocated);
		return 0;
	}
	m_isAllocated[index] = true;
	return index;
}


/**
 * Free an allocated resource.
 * After a resource is no longer needed, for example a destructor is called for a channel assignment
 * class, Free will release the resource value so it can be reused somewhere else in the program.
 */
void Resource::Free(unsigned index)
{
	if (index >= m_size)
	{
		wpi_fatal(IndexOutOfRange);
		return;
	}
	if ( ! m_isAllocated[index] )
	{
		wpi_fatal(NotAllocated);
		return;
	}
	m_isAllocated[index] = false;
}
