#ifndef __SerialPort_h__
#define __SerialPort_h__

/**
 * Driver for the RS-232 serial port on the cRIO.
 * 
 * The current implementation uses the VISA formatted I/O mode.  This means that
 *   all traffic goes through the fomatted buffers.  This allows the intermingled
 *   use of Printf(), Scanf(), and the raw buffer accessors Read() and Write().
 * 
 * More information can be found in the NI-VISA User Manual here:
 *   http://www.ni.com/pdf/manuals/370423a.pdf
 * and the NI-VISA Programmer's Reference Manual here:
 *   http://www.ni.com/pdf/manuals/370132c.pdf
 */
class SerialPort
{
public:
	typedef enum {kParity_None=0, kParity_Odd=1, kParity_Even=2, kParity_Mark=3, kParity_Space=4} Parity;
	typedef enum {kStopBits_One=10, kStopBits_OnePointFive=15, kStopBits_Two=20} StopBits;
	typedef enum {kFlowControl_None=0, kFlowControl_XonXoff=1, kFlowControl_RtsCts=2, kFlowControl_DtrDsr=4} FlowControl;
	typedef enum {kFlushOnAccess=1, kFlushWhenFull=2} WriteBufferMode;

	SerialPort(unsigned int baudRate, unsigned char dataBits = 8, Parity parity = kParity_None, StopBits stopBits = kStopBits_One);
	~SerialPort();
	void SetFlowControl(FlowControl flowControl);
	void EnableTermination(char terminator = '\n');
	void DisableTermination();
	int GetBytesReceived(void);
	void Printf(const char *writeFmt, ...);
	void Scanf(const char *readFmt, ...);
	unsigned int Read(char *buffer, int count);
	unsigned int Write(const char *buffer, int count);
	void SetTimeout(int timeout_ms);
	void SetWriteBufferMode(WriteBufferMode mode);
	void Flush(void);
	void Reset(void);

	/*
	 * Do not call me!
	 */
	//void _internalHandler(unsigned int port, unsigned int eventType, unsigned int event);
private:
	unsigned int m_resourceManagerHandle;
	unsigned int m_portHandle;
};

#endif
