#include "WPILib.h"

SimpleRobot::SimpleRobot(void)
{
	m_robotMainOverridden = true;
}

/**
 * Autonomous should go here.
 * Users should add autonomous code to this method that should run while the field is
 * in the autonomous period.
 */
void SimpleRobot::Autonomous(void)
{
	printf("Provided Autonomous() method running\n");
}

/**
 * Operator control (tele-operated) code should go here.
 * Users should add Operator Control code to this method that should run while the field is
 * in the Operator Control (tele-operated) period.
 */
void SimpleRobot::OperatorControl(void)
{
	printf("Provided OperatorControl() method running\n");
}

/**
 * Robot main program for free-form programs.
 * 
 * This should be overridden by user subclasses if the intent is to not use the Autonomous() and
 * OperatorControl() methods. In that case, the program is responsible for sensing when to run
 * the autonomous and operator control functions in their program.
 * 
 * This method will be called immediately after the constructor is called. If it has not been
 * overridden by a user subclass (i.e. the default version runs), then the Autonomous() and
 * OperatorControl() methods will be called.
 */
void SimpleRobot::RobotMain(void)
{
	printf("Information: No user-supplied RobotMain()\n");
	m_robotMainOverridden = false;
}

/**
 * Start a competition.
 * This code needs to track the order of the field starting to ensure that everything happens
 * in the right order. Currently it waits for the user autonomous code to complete before running
 * the OperatorControl function. Needs to spawn the autonomous code as a task, then kill it after
 * the end of the autonomous period.
 */
void SimpleRobot::StartCompetition(void)
{
	RobotMain();
	if ( ! m_robotMainOverridden )
	{
		// Wait for robot to be enabled
		while (IsDisabled()) Wait(10);
		
		// Now enabled - check if we should run Autonomous code
		if (IsAutonomous())
		{
			Autonomous();
		}
		
		// Wait for Autonomous mode to end
		while (IsAutonomous()) Wait(10);
		
		// Wait for the end of the disabled period between Autonomous and Operator control
		while (IsDisabled()) Wait(10);
		
		// Run the operator control code
		OperatorControl();
	}
}

