#ifndef SOLENOID_H_
#define SOLENOID_H_

#include "SensorBase.h"
#include "ChipObject.h"

/**
 * Solenoid class for running high voltage Digital Output (9472 module).
 * 
 * The Solenoid class is typically used for pneumatics solenoids, but could be used
 * for any device within the current spec of the 9472 module.
 */
class Solenoid : public SensorBase {
public:
	Solenoid(unsigned channel);
	Solenoid(unsigned slot, unsigned channel);
	~Solenoid();
	void Set(bool on);
	bool Get(void);

protected:
	unsigned SlotToIndex(unsigned slot);

private:
	void InitSolenoid();

	unsigned m_chassisSlot; ///< Slot number where the module is plugged into the chassis.
	unsigned m_channel; ///< The channel on the module to control.
	static tSolenoid *m_fpgaSolenoidModule; ///< FPGA Solenoid Module object.
	static unsigned m_refCount; ///< Reference count for the chip object.
};

#endif
