#ifndef TIMER_H_
#define TIMER_H_

#include "taskLib.h"
#include "sysLib.h"
#include "semLib.h"

typedef void (*TimerInterruptHandler)(void *param);

void Wait(unsigned milliseconds);
unsigned GetGameTime(void);
unsigned GetUsClock(void);
unsigned GetMsClock(void);


/**
 * Timer objects measure accumulated time in milliseconds.
 * The timer object functions like a stopwatch. It can be started, stopped, and cleared. When the
 * timer is running its value counts up in milliseconds. When stopped, the timer holds the current
 * value. The implemenation simply records the time when started and subtracts the current time
 * whenever the value is requested.
 */
class Timer
{
public:
	Timer(void);
	virtual ~Timer();
	unsigned long Get(void);
	void Reset(void);
	void Start(void);
	void Stop(void);

private:
	unsigned long m_startTime;
	unsigned long m_accumulatedTime;
	bool m_running;
	SEM_ID m_semaphore;
};

#endif
