#ifndef ACCELEROMETER_H_
#define ACCELEROMETER_H_

#include "SensorBase.h"
#include "AnalogModule.h"
#include "AnalogChannel.h"

/** 
 * Handle operation of the accelerometer.
 * The accelerometer reads acceleration directly through the sensor. Many sensors have
 * multiple axis and can be treated as multiple devices. Each is calibrated by finding
 * the center value over a period of time.
 */
class Accelerometer : public SensorBase {
public:
	Accelerometer(unsigned channel);
	Accelerometer(unsigned slot, unsigned channel);
	Accelerometer(AnalogChannel *channel);
	~Accelerometer(void);

	float GetAcceleration(void);
	void SetSensitivity(float sensitivity);
	void SetZero(float zero);

private:
	void InitAccelerometer(void);

	AnalogChannel *m_analogChannel;
	float m_voltsPerG;
	float m_zeroGVoltage;
	bool m_allocatedChannel;
};

#endif
