#ifndef CPPCOUNTER_H_
#define CPPCOUNTER_H_

#include "AnalogTriggerOutput.h"
#include "ChipObject.h"
#include "CounterBase.h"
#include "SensorBase.h"

/**
 * Class for counting the number of ticks on a digital input channel.
 * This is a general purpose class for counting repetitive events. It can return the number
 * of counts, the period of the most recent cycle, and detect when the signal being counted
 * has stopped by supplying a maximum cycle time.
 */
class Counter : public SensorBase, public CounterBase
{
public:
	typedef enum {kTwoPulse=0, kSemiperiod=1, kPulseLength=2, kExternalDirection=3} Mode;

	Counter(void);
	Counter(unsigned channel);
	Counter(unsigned slot, unsigned channel);
	Counter(DigitalSource *source);
	Counter(AnalogTrigger *trigger);
	virtual ~Counter(void);

	void SetUpSource(unsigned channel);
	void SetUpSource(unsigned slot, unsigned channel);
	void SetUpSource(AnalogTrigger *analogTrigger, AnalogTriggerOutput::Type triggerType);
	void SetUpSource(DigitalSource *source);
	void SetUpSourceEdge(bool risingEdge, bool fallingEdge);
	void ClearUpSource(void);

	void SetDownSource(unsigned channel);
	void SetDownSource(unsigned slot, unsigned channel);
	void SetDownSource(AnalogTrigger *analogTrigger, AnalogTriggerOutput::Type triggerType);
	void SetDownSource(DigitalSource *source);
	void SetDownSourceEdge(bool risingEdge, bool fallingEdge);
	void ClearDownSource(void);

	void SetUpDownCounterMode(void);
	void SetExternalDirectionMode(void);
	void SetSemiPeriodMode(bool highRisingPeriod);
	void SetPulseLengthMode(unsigned threshold);

	// CounterBase interface
	void Start(void);
	signed int Get(void);
	void Reset(void);
	void Stop(void);
	unsigned long GetPeriod(void);
	void SetMaxPeriod(unsigned long maxPeriod);
	bool GetStopped(void);
	bool GetDirection(void);
private:
	void InitCounter(void);

	DigitalSource *m_upSource;		///< What makes the counter count up.
	DigitalSource *m_downSource;	///< What makes the counter count down.
	bool m_allocatedUpSource;		///< Was the upSource allocated locally?
	bool m_allocatedDownSource;	///< Was the downSource allocated locally?
	tCounter *m_counter;				///< The FPGA counter object.
	unsigned m_index;					///< The index of this counter.
};

#endif
