#ifndef GYRO_H_
#define GYRO_H_

/**
 * Use a rate gyro to return the robots heading relative to a starting position.
 * The Gyro class tracks the robots heading based on the starting position. As the robot
 * rotates the new heading is computed by integrating the rate of rotation returned
 * by the sensor. When the class is instantiated, it does a short calibration routine
 * where it samples the gyro while at rest to determine the default offset. This is
 * subtracted from each sample to determine the heading.
 */
class Gyro : public SensorBase
{
public:
	static const unsigned kOversampleBits = 7;
	static const unsigned kAverageBits = 3;
	static const float kSamplesPerSecond = 50.0;

	Gyro(unsigned slot, unsigned channel);
	Gyro(unsigned channel);
	Gyro(AnalogChannel *channel);
	~Gyro(void);
	float GetAngle(void);
	void SetSensitivity(float voltsPerDegreePerSecond);
	void Reset(void);

private:
	void InitGyro();

	AnalogModule *m_module;
	AnalogChannel *m_analog;
	float m_voltsPerDegreePerSecond;
	float m_offset;
	bool m_channelAllocated;
};

#endif
