#include "WPILib.h"

/**
 * Constructor.
 * 
 * @param slot The slot of the digital module that the sensor is plugged into.
 */
HiTechnicCompass::HiTechnicCompass(unsigned slot)
	: m_i2c (NULL)
{
	DigitalModule *module = DigitalModule::GetInstance(slot);
	m_i2c = module->GetI2C(kAddress);

	// Verify Sensor
	const unsigned char kExpectedManufacturer[] = "HiTechnc";
	const unsigned char kExpectedSensorType[] = "Compass ";
	if ( ! m_i2c->VerifySensor(kManufacturerBaseRegister, kManufacturerSize, kExpectedManufacturer) )
	{
		wpi_fatal(CompassManufacturerError);
		return;
	}
	if ( ! m_i2c->VerifySensor(kSensorTypeBaseRegister, kSensorTypeSize, kExpectedSensorType) )
	{
		wpi_fatal(CompassTypeError);
	}
}

/**
 * Destructor.
 */
HiTechnicCompass::~HiTechnicCompass(void)
{
	delete m_i2c;
	m_i2c = NULL;
}

/**
 * Get the compass angle in degrees.
 * 
 * The resolution of this reading is 1 degree.
 * 
 * @return Angle of the compass in degrees.
 */
float HiTechnicCompass::GetAngle(void)
{
	unsigned short heading;
	m_i2c->Read(kHeadingRegister, sizeof(heading), (unsigned char *)&heading);

	// Sensor is little endian... swap bytes
	heading = (heading >> 8) | (heading << 8);

	return (float)heading;
}

