#ifndef I2C_H
#define I2C_H

class DigitalModule;

/**
 * I2C bus interface class.
 * 
 * This class is intended to be used by sensor (and other I2C device) drivers.
 * It probably should not be used directly.
 * 
 * It is constructed by calling DigitalModule::GetI2C() on a DigitalModule object.
 */
class I2C : SensorBase
{
	friend class DigitalModule;
public:
	~I2C(void);
	void Write(unsigned char registerAddress, unsigned char data);
	void Read(unsigned char registerAddress, unsigned char count, unsigned char *data);
	void Broadcast(unsigned char registerAddress, unsigned char data);

	bool VerifySensor(unsigned char registerAddress, unsigned char count, const unsigned char *expected);
private:
	I2C(DigitalModule *module, unsigned char deviceAddress);

	DigitalModule *m_module;
	unsigned char m_deviceAddress;
};

#endif

