#include "WPILib.h"

/**
 * Common initialization code called by all constructors.
 */
void Jaguar::InitJaguar(void)
{
	SetPeriodMultiplier(kPeriodMultiplier_1X);
	SetMaxPositivePwm(kMaxPwm);
	SetMinPositivePwm(kCenterPwm + 1);
	SetCenterPwm(kCenterPwm);
	SetMaxNegativePwm(kCenterPwm - 1);
	SetMinNegativePwm(kMinPwm);
}

/**
 * Constructor that assumes the default digital module.
 * 
 * @param channel The PWM channel on the digital module that the Jaguar is attached to.
 */
Jaguar::Jaguar(unsigned channel) : PWM(channel)
{
	InitJaguar();
}

/**
 * Constructor that specifies the digital module.
 * 
 * @param slot The slot in the chassis that the digital module is plugged into.
 * @param channel The PWM channel on the digital module that the Jaguar is attached to.
 */
Jaguar::Jaguar(unsigned slot, unsigned channel) : PWM(slot, channel)
{
	InitJaguar();
}

Jaguar::~Jaguar(void)
{
}

/**
 * Set the PWM value.  
 * 
 * The PWM value is set using a range of -1.0 to 1.0, appropriately
 * scaling the value for the FPGA.
 * 
 * @param speed The speed value between -1.0 and 1.0 to set.
 */
void Jaguar::Set(float speed)
{
	SetSpeed(speed);
}

/**
 * Get the recently set value of the PWM.
 * 
 * @return The most recently set value for the PWM between -1.0 and 1.0.
 */
float Jaguar::Get(void)
{
	return GetSpeed();
}


