#include "WPILib.h"

/**
 * Common initialization code called by all constructors.
 */
void Servo::InitServo()
{
	SetPeriodMultiplier(kPeriodMultiplier_4X);
	SetMaxPositivePwm(kMaxPwm);
	SetMinNegativePwm(kMinPwm);
}

/**
 * Constructor that assumes the default digital module.
 * 
 * @param channel The PWM channel on the digital module that the servo is attached to.
 */
Servo::Servo(unsigned channel) : PWM(channel)
{
	InitServo();
}

/**
 * Constructor that specifies the digital module.
 * 
 * @param slot The slot in the chassis that the digital module is plugged into.
 * @param channel The PWM channel on the digital module that the servo is attached to.
 */
Servo::Servo(unsigned slot, unsigned channel) : PWM(slot, channel)
{
	InitServo();
}

Servo::~Servo(void)
{
}

/**
 * Set the servo position.
 * 
 * Servo values range from 0.0 to 1.0 that corresponds to full left to full right.
 * 
 * @param value Position from 0.0 to 1.0.
 */
void Servo::Set(float value)
{
	SetPosition(value);
}

/**
 * Get the servo position.
 * 
 * Servo values range from 0.0 to 1.0 that corresponds to full left to full right.
 * 
 * @return Position from 0.0 to 1.0.
 */
float Servo::Get(void)
{
	return GetPosition();
}

/**
 * Set the servo angle.
 * 
 * Assume that the servo angle is linear with respect to the PWM value (big assumption, need to test).
 * 
 * @param degrees The angle in degrees to set the servo.
 */
void Servo::SetAngle(float degrees)
{
	//TODO: what does mod do for negative values?
	degrees = (int)degrees % 360;
	if (degrees < kMinServoAngle)
	{
		degrees = kMinServoAngle;
	}
	else if (degrees > kMaxServoAngle)
	{
		degrees = kMaxServoAngle;
	}

	SetPosition((degrees - kMinServoAngle) / GetServoAngleRange());
}

/**
 * Get the servo angle.
 * 
 * Assume that the servo angle is linear with respect to the PWM value (big assumption, need to test).
s
 * @return The angle in degrees the servo is set to.
 */
float Servo::GetAngle(void)
{
	return (float)GetPosition() * GetServoAngleRange() + kMinServoAngle;
}
