#include <taskLib.h>
#include "Utility.h"
#include "WPIStatus.h"
#include "task.h"
#include <string.h>

Task::Task(char* name, int priority)
{
	m_priority = priority;
	m_taskName = new char[strlen(name) + 1];
	strcpy(m_taskName, name);
}

Task::~Task(void)
{
	delete [] m_taskName;
}

/**
 * Runs this task.
 * If it is already running or unable to start, it fails and returns false.
 */
bool Task::Run(FUNCPTR function, int arg)
{
	m_taskID = taskSpawn(m_taskName,
						m_priority,
						VX_FP_TASK,							// options
						20000,								// stack size
						(FUNCPTR) function,					// function to start
						arg,								// parameter 1 - pointer to this class
						0, 0, 0, 0, 0, 0, 0, 0, 0);			// additional unused parameters
	if (m_taskID == ERROR)
	{
		wpi_fatal(TaskError);
		return false;
	}
	return true;
}

/**
 * Restarts a running task.
 * If the task isn't started, it starts it.
 * If the task is running and we are unable to kill the previous instance, returns false.
 */
bool Task::Restart()
{
	taskRestart(m_taskID);
	return false;
}

/**
 * Kills the running task.
 * Returns true on success false if the task doesn't exist or we are unable to kill it.
 */
bool Task::Delete()
{
	taskDelete(m_taskID);
	return false;
}

/**
 * Pauses a running task.
 * Returns true on success, false if unable to pause or the task isn't running.
 */
bool Task::Suspend()
{
	taskSuspend(m_taskID);
	return false;
}

/**
 * Resumes a paused task.
 * Returns true on success, false if unable to resume or if the task isn't running/paused.
 */
bool Task::Resume()
{
	taskResume(m_taskID);
	return false;
}

/**
 * Verifies a task is running.
 */
bool Task::Verify()
{
	if (taskIdVerify(m_taskID) == OK)
		return true;
	else
		return false;
}

/**
 * Gets the priority of a task.
 * Range: ? to ???
 */
int Task::GetPriority()
{
	return m_priority;
}

/**
 * Sets the priority of the task.
 * Range: ? to ???
 */
void Task::SetPriority(int p)
{
	m_priority = p;
	taskPrioritySet(m_taskID, m_priority);
}

/**
 * Returns the name of the task.
 */
char* Task::GetName()
{
	return m_taskName;
}

/**
 * Returns the task ID of this task.
 */
int Task::GetID()
{
	return m_taskID;
}

