#ifndef ULTRASONIC_H_
#define ULTRASONIC_H_

#include "SensorBase.h"

class Counter;
class DigitalInput;
class DigitalOutput;

/**
 * Ultrasonic rangefinder class.
 * The Ultrasonic rangefinder measures absolute distance based on the round-trip time
 * of a ping generated by the controller. These sensors use two transducers, a speaker and
 * a microphone both tuned to the ultrasonic range. A common ultrasonic sensor, the Daventech SRF04
 * requires a short pulse to be generated on a digital channel. This causes the chirp to be
 * emmitted. A second line becomes high as the ping is transmitted and goes low when
 * the echo is received. The time that the line is high determines the round trip distance
 * (time of flight).
 */
class Ultrasonic: public SensorBase
{
public:
	Ultrasonic(DigitalOutput *pingChannel, DigitalInput *echoChannel);
	Ultrasonic(unsigned pingChannel, unsigned echoChannel);
	Ultrasonic::Ultrasonic(unsigned pingSlot, unsigned pingChannel,
							unsigned echoSlot, unsigned echoChannel);
	~Ultrasonic(void);

	void Ping(void);
	bool IsRangeValid(void);
	void SetAutomaticMode(bool enabling);
	unsigned long GetRangeInches(void);
	unsigned long GetRangeMM(void);
	bool IsEnabled(void) { return m_enabled; }
	void SetEnabled(bool enable) { m_enabled = enable; }

private:
	void Initialize(void);

	static void ultrasonicChecker();

	static const unsigned kPingTime = 10;	///< Time (uS) for the ping trigger pulse.
	static const unsigned kPriority = 90;	///< Priority that the ultrasonic round robin task runs.
	static const unsigned kMaxUltrasonicTime = 100;	///< Max time (ms) between readings.
	static const unsigned kSpeedOfSoundInchesPerSec = 1130 * 12;
	DigitalInput *m_echoChannel;
	DigitalOutput *m_pingChannel;
	bool m_allocatedChannels;
	bool m_enabled;
	Counter *m_counter;
	Ultrasonic *m_nextSensor;
};

#endif

