
#include "Victor.h"

/**
 * Common initialization code called by all constructors.
 */
void Victor::InitVictor(void)
{
	SetPeriodMultiplier(kPeriodMultiplier_2X);
	SetMaxPositivePwm(kMaxPwm);
	SetCenterPwm(kCenterPwm);
	SetMinNegativePwm(kMinPwm);
	EnableDeadbandElimination(false);
}

/**
 * Optionally eliminate the deadband from the Victors.
 * @param eliminateDeadband If true, set the motor curve on the Victor to eliminate
 * the deadband in the middle of the range. Otherwise, keep the full range without
 * modifying any values.
 */
void Victor::EnableDeadbandElimination(bool eliminateDeadband)
{
	if (eliminateDeadband)
	{
		SetMinPositivePwm(kDeadbandMaxPwm);
		SetMaxNegativePwm(kDeadbandMinPwm);
	}
	else
	{
		SetMinPositivePwm(kCenterPwm + 1);
		SetMaxNegativePwm(kCenterPwm - 1);
	}
}

/**
 * Constructor that assumes the default digital module.
 * 
 * @param channel The PWM channel on the digital module that the Victor is attached to.
 */
Victor::Victor(unsigned channel) : PWM(channel)
{
	InitVictor();
}

/**
 * Constructor that specifies the digital module.
 * 
 * @param slot The slot in the chassis that the digital module is plugged into.
 * @param channel The PWM channel on the digital module that the Victor is attached to.
 */
Victor::Victor(unsigned slot, unsigned channel) : PWM(slot, channel)
{
	InitVictor();
}

Victor::~Victor(void)
{
}

/**
 * Set the PWM value.  
 * 
 * The PWM value is set using a range of -1.0 to 1.0, appropriately
 * scaling the value for the FPGA.
 * 
 * @param speed The speed value between -1.0 and 1.0 to set.
 */
void Victor::Set(float speed)
{
	SetSpeed(speed);
}

/**
 * Get the recently set value of the PWM.
 * 
 * @return The most recently set value for the PWM between -1.0 and 1.0.
 */
float Victor::Get()
{
	return GetSpeed();
}

