#ifndef WATCHDOG_H
#define WATCHDOG_H

#include "ChipObject.h"
#include "SensorBase.h"

/**
 * Watchdog timer class.
 * The watchdog timer is designed to keep the robots safe. The idea is that the robot program must
 * constantly "feed" the watchdog otherwise it will shut down all the motor outputs. That way if a
 * program breaks, rather than having the robot continue to operate at the last known speed, the
 * motors will be shut down.
 * 
 * This is serious business.  Don't just disable the watchdog.  You can't afford it!
 * 
 * http://thedailywtf.com/Articles/_0x2f__0x2f_TODO_0x3a__Uncomment_Later.aspx
 */
class Watchdog : public SensorBase
{
public:
	static const unsigned int kDefaultWatchdogExpiration_ms = 500;

	Watchdog(void);
	~Watchdog(void);
	void Feed(void);
	void Kill(void);
	unsigned int GetTimer(void);
	unsigned int GetExpiration(void);
	void SetExpiration(unsigned int expiration_ms);
	bool GetEnabled(void);
	void SetEnabled(bool enabled);
	bool IsAlive(void);
	bool IsSystemActive(void);

private:
	tWatchdog *m_fpgaWatchDog;
};

#endif
