/********************************************************************************
*
*  Project   		: FIRST Motor Controller
*  Category 		: Team Deliverable Software (as source code) 	
*  Subsystem 		: FRC Vision for cRIO     			
* 
*  File Name  		: baefrc_utilities.h          
*  Contributors   	: JDG, ELF
*  Creation Date 	: August 12, 2008
*  Revision History	: Source code and revision history are maintained at
*                               	   sourceforge.WPI.edu  
*  
*  File Description	: Globally defined values for the FIRST Vision API
* 
*  API: Because the nivision.h uses C++ style comments, any file including this
*  must be a .cpp (not .c) - unless we find a compiler directive to override the
*  comment processing or to compile C as C++.
*                              
*/
/*----------------------------------------------------------------------------*/
/*        Copyright (c) FIRST 2008.  All Rights Reserved.                     */
/*     Open Source Software - may be modified and shared by FRC teams.        */
/*   Must be accompanied by the BSD license file in $(WIND_BASE)/WPILib.      */
/*----------------------------------------------------------------------------*/

#ifndef baefrc_utilities_h
#define baefrc_utilities_h

#include "frcvision.h"

/*  Constants */

#define LOG_DEBUG    __FILE__,funcName,__LINE__,DEBUG_TYPE
#define LOG_INFO     __FILE__,funcName,__LINE__,INFO_TYPE
#define LOG_ERROR    __FILE__,funcName,__LINE__,ERROR_TYPE
#define LOG_CRITICAL __FILE__,funcName,__LINE__,CRITICAL_TYPE
#define LOG_FATAL    __FILE__,funcName,__LINE__,FATAL_TYPE
#define LOG_DEBUG    __FILE__,funcName,__LINE__,DEBUG_TYPE

/* Error Codes */
#define ERR_frcGENERAL_ERROR				166000	// 
#define ERR_frcCOLOR_NOT_FOUND				166100	// frcvtrack.cpp
#define ERR_frcPARTICLE_TOO_SMALL			166101	// frcvtrack.cpp

#define ERR_CAMERA_FAILURE					166200	// frcAxis.cpp
#define ERR_CAMERA_SOCKET_CREATE_FAILED		166201	// frcAxis.cpp
#define ERR_CAMERA_CONNECT_FAILED			166202	// frcAxis.cpp
#define ERR_CAMERA_STALE_IMAGE				166203	// frcAxis.cpp
#define ERR_CAMERA_NOT_INITIALIZED			166204	// frcAxis.cpp
#define ERR_CAMERA_NO_BUFFER_AVAILABLE		166205	// frcAxis.cpp
#define ERR_CAMERA_HEADER_ERROR				166206	// frcAxis.cpp
#define ERR_CAMERA_BLOCKING_TIMEOUT			166207	// frcAxis.cpp
#define ERR_CAMERA_AUTHORIZATION_FAILED		166208	// frcAxis.cpp
#define ERR_CAMERA_TASK_SPAWN_FAILED		166209	// frcAxis.cpp
#define ERR_CAMERA_TASK_INPUT_OUT_OF_RANGE	166210	// frcAxis.cpp

/*   Enumerated Types */

/** debug levels */
enum dprint_type {DEBUG_TYPE, INFO_TYPE, ERROR_TYPE, CRITICAL_TYPE, FATAL_TYPE};

/** debug output setting */
typedef enum FrcDebugOutputType_enum { 
	DEBUG_OFF, DEBUG_MOSTLY_OFF, DEBUG_SCREEN_ONLY, DEBUG_FILE_ONLY, DEBUG_SCREEN_AND_FILE
}FrcDebugOutputType;

/*  Enumerated Types */

/** Predefined hues */
typedef enum FrcHue_enum { 
	// Basic colors
	RED, GREEN, BLUE, YELLOW, ORANGE, PURPLE, WHITE,
	// User defined hues
	HUE1, HUE2
}FrcHue;


/** Predefined saturation / luminance settings */
typedef enum FrcLightLevel_enum { 
	PASSIVE_LIGHT, BRIGHT_LIGHT, ACTIVE_LIGHT, WHITE_LIGHT
}FrcLightLevel;

/* Utility functions */

/* debug */
void frcSetDebugFlag ( FrcDebugOutputType flag  ); 
void dprintf ( char * tempString, ...  );  /* Variable argument list */

/* error handling */
int frcGetLastError();
char* frcGetErrorText(int errorCode);

/* color tracking support functions */
int frcSetHueRange(FrcHue color, Range* hueRange);
int frcSetLightLevel(FrcLightLevel light, Range* satRange, Range* lumRange);
void frcPrintReport(FrcParticleAnalysisReport* myReport);
void frcPrintReport(FrcColorReport* myReport);

/* set FRC ranges for drive */
double frcRangeToNormalized(double pixel, int range);
/* change normalized value to any range - used for servo */
float frcNormalizeToRange(float normalizedValue, float minRange, float maxRange);
float frcNormalizeToRange(float normalizedValue);

/* system utilities */
double frcTime ( void  );
double frcElapsedTime ( double startTime  ); 
void frcSleep ( double sleepTime );
void frcShowActivity (char *fmt, ...);

#endif

