/********************************************************************************
*
*  Project   		: FIRST Motor Controller
*  Category 		: Team Deliverable Software (as source code) 	
*  Subsystem 		: FRC Vision for cRIO    			
* 
*  File Name  		: frcvAxis.h          
*  Contributors   	: ELF
*  Creation Date 	: August 12, 2008
*  Revision History	: Source code and revision history are maintained at
*                               	   sourceforge.WPI.edu  
*  
*  File Description	: Globally defined values for the FIRST Vision API
* 
*  API: Because the nivision.h uses C++ style comments, any file including this
*  must be a .cpp (not .c) - unless we find a compiler directive to override the
*  comment processing or to compile C as C++.
*                              
*/
/*----------------------------------------------------------------------------*/
/*        Copyright (c) FIRST 2008.  All Rights Reserved.                     */
/*  Open Source Software - may be modified and shared by FRC teams. The code  */
/*  must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib. */
/*----------------------------------------------------------------------------*/

#ifndef frcvAxis_h
#define frcvAxis_h

#include "nivision.h"

/*  Enumerated Types */
/** @brief Counters for camera metrics */
enum FrcvCameraMetric {CAM_STARTS, CAM_STOPS, 
	CAM_NUM_IMAGE, CAM_BUFFERS_WRITTEN, CAM_BLOCKING_COUNT,
	
	CAM_SOCKET_OPEN, CAM_SOCKET_INIT_ATTEMPTS, CAM_BLOCKING_TIMEOUT,
	CAM_GETIMAGE_SUCCESS, CAM_GETIMAGE_FAILURE, 
	
	CAM_STALE_IMAGE, CAM_GETIMAGE_BEFORE_INIT, CAM_GETIMAGE_BEFORE_AVAILABLE,
	CAM_READ_JPEG_FAILURE, CAM_PID_SIGNAL_ERR, 
	
	CAM_BAD_IMAGE_SIZE, CAM_HEADER_ERROR};

#define CAM_NUM_METRICS 17

/**
@brief Possible image sizes that you can set on the camera.
*/
enum ImageSize {
    k640x480,
    k320x240,
    k160x120,
};

/**
@brief Possible rotation values that you can set on the camera.
*/
enum ImageRotation { ROT_0 = 0, ROT_90 = 90, ROT_180 = 180, ROT_270 = 270 };


/*  Image Acquisition functions */

/* obtains an image from the camera server */
int frcGetImage(Image* cameraImage, double *timestamp);
int frcGetImageBlocking(Image* cameraImage, double *timestamp, double lastImageTimestamp);
/* obtains raw image string to send to PC */
int frcGetImageDataBlocking(char** imageData, int* numBytes, double* timestamp, double lastImageTimestamp);

/* start the camera server */
void frcStartImageAcquisition(void);
void frcStopImageAcquisition(void);
void frcStartImageSignal(int taskId);

/* status & metrics */
int frcCameraInitialized();
int frcGetCameraMetric(FrcvCameraMetric metric);

/* camera task control */
int frcStartCameraTask();
int frcStartCameraTask(ImageRotation rotation);
int frcStopCameraTask();

#endif

